@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +23,9 @@
 #endif
 
 #include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/Xatom.h>
+#include <X11/extensions/shape.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -33,6 +35,8 @@
 #include <libxfce4util/debug.h>
 #include <libxfce4util/i18n.h>
 #include <libxfcegui4/libxfcegui4.h>
+#include <string.h>
+
 #include "main.h"
 #include "misc.h"
 #include "workspaces.h"
@@ -85,14 +89,14 @@ typedef enum
 }
 XfwmButtonClickType;
 
-static inline XfwmButtonClickType
+static XfwmButtonClickType
 typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
 {
     unsigned int button;
     int xcurrent, ycurrent, x, y, total;
     int g = GrabSuccess;
     int clicks;
-    Time t0, t1;
+    Time t0;
 
     g_return_val_if_fail (ev != NULL, XFWM_BUTTON_UNDEFINED);
     g_return_val_if_fail (w != None, XFWM_BUTTON_UNDEFINED);
@@ -110,19 +114,22 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
     button = ev->xbutton.button;
     x = xcurrent = ev->xbutton.x_root;
     y = ycurrent = ev->xbutton.y_root;
-    t0 = CurrentTime;
+    t0 = GDK_CURRENT_TIME;
     total = 0;
     clicks = 1;
 
     while ((ABS (x - xcurrent) < 2) && (ABS (y - ycurrent) < 2)
         && (total < params.dbl_click_time)
-        && ((CurrentTime - t0) < params.dbl_click_time))
+        && ((GDK_CURRENT_TIME - t0) < params.dbl_click_time))
     {
         g_usleep (10000);
         total += 10;
+        if (XCheckMaskEvent (dpy, FocusChangeMask, ev))
+        {
+            handleEvent (ev);
+        }
         if (XCheckMaskEvent (dpy, ButtonReleaseMask | ButtonPressMask, ev))
         {
-            last_timestamp = stashEventTime (last_timestamp, ev);
             if (ev->xbutton.button == button)
             {
                 clicks++;
@@ -136,7 +143,6 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
         }
         if (XCheckMaskEvent (dpy, ButtonMotionMask | PointerMotionMask, ev))
         {
-            last_timestamp = stashEventTime (last_timestamp, ev);
             xcurrent = ev->xmotion.x_root;
             ycurrent = ev->xmotion.y_root;
         }
@@ -146,7 +152,7 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
     return (XfwmButtonClickType) clicks;
 }
 
-static inline void
+static void
 clear_timeout (void)
 {
     if (raise_timeout)
@@ -156,7 +162,7 @@ clear_timeout (void)
     }
 }
 
-static inline gboolean
+static gboolean
 raise_cb (gpointer data)
 {
     Client *c = NULL;
@@ -172,7 +178,7 @@ raise_cb (gpointer data)
     return (TRUE);
 }
 
-static inline void
+static void
 reset_timeout (void)
 {
     if (raise_timeout)
@@ -184,7 +190,7 @@ reset_timeout (void)
         NULL, NULL);
 }
 
-static inline void
+static void
 moveRequest (Client * c, XEvent * ev)
 {
     if (CLIENT_FLAG_TEST_AND_NOT (c, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
@@ -193,10 +199,10 @@ moveRequest (Client * c, XEvent * ev)
     }
 }
 
-static inline void
+static void
 resizeRequest (Client * c, int corner, XEvent * ev)
 {
-    clientSetFocus (c, FOCUS_SORT);
+    clientSetFocus (c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
 
     if (CLIENT_FLAG_TEST_ALL (c,
             CLIENT_FLAG_HAS_RESIZE | CLIENT_FLAG_IS_RESIZABLE))
@@ -209,7 +215,7 @@ resizeRequest (Client * c, int corner, XEvent * ev)
     }
 }
 
-static inline void
+static void
 spawn_shortcut (int i)
 {
     GError *error = NULL;
@@ -228,7 +234,7 @@ spawn_shortcut (int i)
     }
 }
 
-static inline void
+static void
 handleMotionNotify (XMotionEvent * ev)
 {
     int msx, msy, max;
@@ -264,14 +270,12 @@ handleMotionNotify (XMotionEvent * ev)
                 workspaceSwitch (workspace + 1, NULL);
             }
             while (XCheckWindowEvent(dpy, ev->window, PointerMotionMask, (XEvent *) ev))
-            {
-                last_timestamp = stashEventTime (last_timestamp, (XEvent *) ev);
-            }    
+                ; /* void */
         }
     }
 }
 
-static inline void
+static void
 handleKeyPress (XKeyEvent * ev)
 {
     Client *c = NULL;
@@ -318,7 +322,7 @@ handleKeyPress (XKeyEvent * ev)
                 clientClose (c);
                 break;
             case KEY_HIDE_WINDOW:
-                if (CLIENT_CAN_HIDE_WINDOW (c))
+                if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER) && CLIENT_CAN_HIDE_WINDOW (c))
                 {
                     clientHide (c, c->win_workspace, TRUE);
                 }
@@ -336,8 +340,11 @@ handleKeyPress (XKeyEvent * ev)
                 clientToggleShaded (c);
                 break;
             case KEY_STICK_WINDOW:
-                clientToggleSticky (c, TRUE);
-                frameDraw (c, FALSE, FALSE);
+                if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER) && CLIENT_CAN_STICK_WINDOW (c))
+                {
+                    clientToggleSticky (c, TRUE);
+                    frameDraw (c, FALSE, FALSE);
+                }
                 break;
             case KEY_RAISE_WINDOW:
                 clientRaise (c);
@@ -354,31 +361,16 @@ handleKeyPress (XKeyEvent * ev)
                 workspaceSwitch (workspace - 1, c);
                 break;
             case KEY_MOVE_WORKSPACE_1:
-                workspaceSwitch (0, c);
-                break;
             case KEY_MOVE_WORKSPACE_2:
-                workspaceSwitch (1, c);
-                break;
             case KEY_MOVE_WORKSPACE_3:
-                workspaceSwitch (2, c);
-                break;
             case KEY_MOVE_WORKSPACE_4:
-                workspaceSwitch (3, c);
-                break;
             case KEY_MOVE_WORKSPACE_5:
-                workspaceSwitch (4, c);
-                break;
             case KEY_MOVE_WORKSPACE_6:
-                workspaceSwitch (5, c);
-                break;
             case KEY_MOVE_WORKSPACE_7:
-                workspaceSwitch (6, c);
-                break;
             case KEY_MOVE_WORKSPACE_8:
-                workspaceSwitch (7, c);
-                break;
             case KEY_MOVE_WORKSPACE_9:
-                workspaceSwitch (8, c);
+                clientRaise (c);
+                workspaceSwitch (key - KEY_MOVE_WORKSPACE_1, c);
                 break;
             default:
                 break;
@@ -413,61 +405,27 @@ handleKeyPress (XKeyEvent * ev)
             workspaceSetCount (params.workspace_count - 1);
             break;
         case KEY_WORKSPACE_1:
-            workspaceSwitch (0, NULL);
-            break;
         case KEY_WORKSPACE_2:
-            workspaceSwitch (1, NULL);
-            break;
         case KEY_WORKSPACE_3:
-            workspaceSwitch (2, NULL);
-            break;
         case KEY_WORKSPACE_4:
-            workspaceSwitch (3, NULL);
-            break;
         case KEY_WORKSPACE_5:
-            workspaceSwitch (4, NULL);
-            break;
         case KEY_WORKSPACE_6:
-            workspaceSwitch (5, NULL);
-            break;
         case KEY_WORKSPACE_7:
-            workspaceSwitch (6, NULL);
-            break;
         case KEY_WORKSPACE_8:
-            workspaceSwitch (7, NULL);
-            break;
         case KEY_WORKSPACE_9:
-            workspaceSwitch (8, NULL);
+            workspaceSwitch (key - KEY_WORKSPACE_1, NULL);
             break;
         case KEY_SHORTCUT_1:
-            spawn_shortcut (0);
-            break;
         case KEY_SHORTCUT_2:
-            spawn_shortcut (1);
-            break;
         case KEY_SHORTCUT_3:
-            spawn_shortcut (2);
-            break;
         case KEY_SHORTCUT_4:
-            spawn_shortcut (3);
-            break;
         case KEY_SHORTCUT_5:
-            spawn_shortcut (4);
-            break;
         case KEY_SHORTCUT_6:
-            spawn_shortcut (5);
-            break;
         case KEY_SHORTCUT_7:
-            spawn_shortcut (6);
-            break;
         case KEY_SHORTCUT_8:
-            spawn_shortcut (7);
-            break;
         case KEY_SHORTCUT_9:
-            spawn_shortcut (8);
-            break;
         case KEY_SHORTCUT_10:
-            spawn_shortcut (9);
+            spawn_shortcut (key - KEY_SHORTCUT_1);
             break;
         default:
             break;
@@ -479,7 +437,7 @@ handleKeyPress (XKeyEvent * ev)
  * Button 2 : Move
  * Button 3 : Resize
  */
-static inline void
+static void
 edgeButton (Client * c, int part, XButtonEvent * ev)
 {
     if (ev->button == Button2)
@@ -512,19 +470,20 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
     }
 }
 
-static inline void
+static void
 button1Action (Client * c, XButtonEvent * ev)
 {
-    XEvent copy_event = (XEvent) * ev;
+    XEvent copy_event;
     XfwmButtonClickType tclick;
 
     g_return_if_fail (c != NULL);
     g_return_if_fail (ev != NULL);
 
-    clientSetFocus (c, FOCUS_SORT);
+    clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
     clientRaise (c);
     clientPassGrabButton1 (c);
 
+    memcpy(&copy_event, ev, sizeof(XEvent));
     tclick = typeOfClick (c->frame, &copy_event, TRUE);
 
     if ((tclick == XFWM_BUTTON_DRAG)
@@ -552,7 +511,7 @@ button1Action (Client * c, XButtonEvent * ev)
     }
 }
 
-static inline void
+static void
 titleButton (Client * c, int state, XButtonEvent * ev)
 {
     g_return_if_fail (c != NULL);
@@ -574,9 +533,10 @@ titleButton (Client * c, int state, XButtonEvent * ev)
            for gtk to handle it (in case we open up the menu)
          */
 
-        XEvent copy_event = (XEvent) * ev;
+        XEvent copy_event;
         XfwmButtonClickType tclick;
 
+        memcpy(&copy_event, ev, sizeof(XEvent));
         tclick = typeOfClick (c->frame, &copy_event, FALSE);
 
         if (tclick == XFWM_BUTTON_DRAG)
@@ -585,7 +545,7 @@ titleButton (Client * c, int state, XButtonEvent * ev)
         }
         else
         {
-            clientSetFocus (c, FOCUS_SORT);
+            clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
             if (params.raise_on_click)
             {
                 clientRaise (c);
@@ -622,20 +582,11 @@ titleButton (Client * c, int state, XButtonEvent * ev)
     }
 }
 
-static inline void
+static void
 rootScrollButton (XButtonEvent * ev)
 {
     static Time lastscroll = (Time) 0;
-    XEvent otherEvent;
 
-    while (XCheckTypedWindowEvent (dpy, root, ButtonPress, &otherEvent))
-    {
-        last_timestamp = stashEventTime (last_timestamp, &otherEvent);
-        if (otherEvent.xbutton.button != ev->button)
-        {
-            XPutBackEvent (dpy, &otherEvent);
-        }
-    }
     if ((ev->time - lastscroll) < 100)  /* ms */
     {
         /* Too many events in too little time, drop this event... */
@@ -653,7 +604,7 @@ rootScrollButton (XButtonEvent * ev)
 }
 
 
-static inline void
+static void
 handleButtonPress (XButtonEvent * ev)
 {
     Client *c = NULL;
@@ -703,7 +654,7 @@ handleButtonPress (XButtonEvent * ev)
         {
             if (ev->button <= Button3)
             {
-                clientSetFocus (c, FOCUS_SORT);
+                clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
                 if (params.raise_on_click)
                 {
                     clientRaise (c);
@@ -725,9 +676,10 @@ handleButtonPress (XButtonEvent * ev)
                    for gtk to handle it (in case we open up the menu)
                  */
 
-                XEvent copy_event = (XEvent) * ev;
+                XEvent copy_event;
                 XfwmButtonClickType tclick;
 
+                memcpy(&copy_event, ev, sizeof(XEvent));
                 tclick = typeOfClick (c->frame, &copy_event, TRUE);
 
                 if (tclick == XFWM_BUTTON_DOUBLE_CLICK)
@@ -736,7 +688,7 @@ handleButtonPress (XButtonEvent * ev)
                 }
                 else
                 {
-                    clientSetFocus (c, FOCUS_SORT);
+                    clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
                     if (params.raise_on_click)
                     {
                         clientRaise (c);
@@ -806,7 +758,7 @@ handleButtonPress (XButtonEvent * ev)
                 {
                     clientPassGrabButton1 (c);
                 }
-                clientSetFocus (c, FOCUS_SORT);
+                clientSetFocus (c, ev->time, NO_FOCUS_FLAG);
                 if ((params.raise_on_click) || !CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_BORDER))
                 {
                     clientRaise (c);
@@ -835,13 +787,13 @@ handleButtonPress (XButtonEvent * ev)
     }
     else
     {
-        XUngrabPointer (dpy, CurrentTime);
+        XUngrabPointer (dpy, GDK_CURRENT_TIME);
         XSendEvent (dpy, gnome_win, FALSE, SubstructureNotifyMask,
             (XEvent *) ev);
     }
 }
 
-static inline void
+static void
 handleButtonRelease (XButtonEvent * ev)
 {
     TRACE ("entering handleButtonRelease");
@@ -849,7 +801,7 @@ handleButtonRelease (XButtonEvent * ev)
     XSendEvent (dpy, gnome_win, FALSE, SubstructureNotifyMask, (XEvent *) ev);
 }
 
-static inline void
+static void
 handleDestroyNotify (XDestroyWindowEvent * ev)
 {
     Client *c = NULL;
@@ -873,7 +825,7 @@ handleDestroyNotify (XDestroyWindowEvent * ev)
     }
 }
 
-static inline void
+static void
 handleMapRequest (XMapRequestEvent * ev)
 {
     Client *c = NULL;
@@ -910,7 +862,7 @@ handleMapRequest (XMapRequestEvent * ev)
     }
 }
 
-static inline void
+static void
 handleMapNotify (XMapEvent * ev)
 {
     Client *c = NULL;
@@ -929,7 +881,7 @@ handleMapNotify (XMapEvent * ev)
     }
 }
 
-static inline void
+static void
 handleUnmapNotify (XUnmapEvent * ev)
 {
     Client *c = NULL;
@@ -995,7 +947,7 @@ handleUnmapNotify (XUnmapEvent * ev)
     }
 }
 
-static inline void
+static void
 handleConfigureNotify (XConfigureEvent * ev)
 {
     TRACE ("entering handleConfigureNotify");
@@ -1014,7 +966,7 @@ handleConfigureNotify (XConfigureEvent * ev)
     }
 }
 
-static inline void
+static void
 handleConfigureRequest (XConfigureRequestEvent * ev)
 {
     Client *c = NULL;
@@ -1027,7 +979,6 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
     /* Compress events - logic taken from kwin */
     while (XCheckTypedWindowEvent (dpy, ev->window, ConfigureRequest, &otherEvent))
     {
-        last_timestamp = stashEventTime (last_timestamp, &otherEvent);
         if (otherEvent.xconfigurerequest.value_mask == ev->value_mask)
         {
             ev = &otherEvent.xconfigurerequest;
@@ -1139,8 +1090,8 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
         }
         if (ev->value_mask & CWStackMode)
         {
-	    clientPassGrabButton1 (NULL);
-	}
+            clientPassGrabButton1 (NULL);
+        }
 #if 0
         /* Let's say that if the client performs a XRaiseWindow, we show the window if hidden */
         if ((ev->value_mask & CWStackMode) && (wc.stack_mode == Above))
@@ -1165,7 +1116,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
     }
 }
 
-static inline void
+static void
 handleEnterNotify (XCrossingEvent * ev)
 {
     Client *c = NULL;
@@ -1185,9 +1136,9 @@ handleEnterNotify (XCrossingEvent * ev)
     if (c && !(params.click_to_focus) && (clientAcceptFocus (c)))
     {
         TRACE ("EnterNotify window is \"%s\"", c->name);
-        if ((c->type != WINDOW_DOCK) && (c->type != WINDOW_DESKTOP))
+        if (!(c->type & (WINDOW_DOCK | WINDOW_DESKTOP)))
         {
-            clientSetFocus (c, FOCUS_SORT);
+            clientSetFocus (c, ev->time, FOCUS_FORCE);
             if (!(params.raise_on_click))
             {
                 clientPassGrabButton1 (c);
@@ -1196,7 +1147,7 @@ handleEnterNotify (XCrossingEvent * ev)
     }
 }
 
-static inline void
+static void
 handleLeaveNotify (XCrossingEvent * ev)
 {
     TRACE ("entering handleLeaveNotify");
@@ -1215,7 +1166,7 @@ handleLeaveNotify (XCrossingEvent * ev)
     }
 }
 
-static inline void
+static void
 handleFocusIn (XFocusChangeEvent * ev)
 {
     Client *c = NULL;
@@ -1258,12 +1209,7 @@ handleFocusIn (XFocusChangeEvent * ev)
         c = clientGetFocus ();
         if (c)
         {
-            clientSetFocus (c, FOCUS_FORCE);
-        }
-        else
-        {
-            clientSetFocus (NULL, FOCUS_NONE);
-            clientPassGrabButton1 (NULL);
+            clientSetFocus (c, GDK_CURRENT_TIME, FOCUS_FORCE);
         }
         return;
     }
@@ -1279,7 +1225,7 @@ handleFocusIn (XFocusChangeEvent * ev)
     if (c)
     {
         TRACE ("focus set to \"%s\" (0x%lx)", c->name, c->window);
-        clientUpdateFocus (c, FOCUS_NONE);
+        clientUpdateFocus (c, FOCUS_SORT);
         if (params.raise_on_focus && !params.click_to_focus)
         {
             reset_timeout ();
@@ -1287,7 +1233,7 @@ handleFocusIn (XFocusChangeEvent * ev)
     }
 }
 
-static inline void
+static void
 handleFocusOut (XFocusChangeEvent * ev)
 {
     Client *c = NULL;
@@ -1329,7 +1275,7 @@ handleFocusOut (XFocusChangeEvent * ev)
         if ((c) && (c == clientGetFocus ()))
         {
             TRACE ("focus lost from \"%s\" (0x%lx)", c->name, c->window);
-            clientUpdateFocus (NULL, FOCUS_NONE);
+            clientUpdateFocus (NULL, NO_FOCUS_FLAG);
             clientPassGrabButton1 (NULL);
             /* Clear timeout */
             clear_timeout ();
@@ -1337,7 +1283,7 @@ handleFocusOut (XFocusChangeEvent * ev)
     }
 }
 
-static inline void
+static void
 handlePropertyNotify (XPropertyEvent * ev)
 {
     Client *c = NULL;
@@ -1442,7 +1388,7 @@ handlePropertyNotify (XPropertyEvent * ev)
     }
 }
 
-static inline void
+static void
 handleClientMessage (XClientMessageEvent * ev)
 {
     Client *c = NULL;
@@ -1556,7 +1502,7 @@ handleClientMessage (XClientMessageEvent * ev)
             clientSetWorkspace (c, workspace, TRUE);
             clientShow (c, TRUE);
             clientRaise (c);
-            clientSetFocus (c, FOCUS_SORT);
+            clientSetFocus (c, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
             clientPassGrabButton1 (c);
         }
     }
@@ -1598,7 +1544,7 @@ handleClientMessage (XClientMessageEvent * ev)
     }
 }
 
-static inline void
+static void
 handleShape (XShapeEvent * ev)
 {
     Client *c = NULL;
@@ -1612,7 +1558,7 @@ handleShape (XShapeEvent * ev)
     }
 }
 
-static inline void
+static void
 handleColormapNotify (XColormapEvent * ev)
 {
     Client *c = NULL;
@@ -1635,7 +1581,6 @@ handleEvent (XEvent * ev)
     TRACE ("entering handleEvent");
 
     sn_process_event (ev);
-    last_timestamp = stashEventTime (last_timestamp, ev);
     switch (ev->type)
     {
         case MotionNotify:
@@ -1867,7 +1812,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY))
         {
             ops |= MENU_OP_UNSTICK;
-            if (!CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_STICK))
+            if (!CLIENT_CAN_STICK_WINDOW(c))
             {
                 insensitive |= MENU_OP_UNSTICK;
             }
@@ -1875,7 +1820,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
         else
         {
             ops |= MENU_OP_STICK;
-            if (!CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_STICK))
+            if (!CLIENT_CAN_STICK_WINDOW(c))
             {
                 insensitive |= MENU_OP_STICK;
             }
