@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -834,7 +834,7 @@ checkKdeSystrayWindow(Display * dpy, Window window)
     Window trayIconForWindow;
 
     TRACE ("entering GetWindowRole");
-    g_return_if_fail (window != None);
+    g_return_val_if_fail (window != None, FALSE);
     
     XGetWindowProperty(dpy, window, kde_net_wm_system_tray_window_for, 0L, 
             sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format, 
@@ -900,12 +900,15 @@ getWindowRole (Display * dpy, Window window, char **role)
 
     if (XGetTextProperty (dpy, window, &tp, wm_window_role))
     {
-        if ((tp.value) && (tp.encoding == XA_STRING) && (tp.format == 8)
-            && (tp.nitems != 0))
+        if (tp.value)
         {
-            *role = strdup ((char *) tp.value);
+            if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
+            {
+                *role = strdup ((char *) tp.value);
+                XFree (tp.value);
+                return TRUE;
+            }
             XFree (tp.value);
-            return TRUE;
         }
     }
 
@@ -1007,12 +1010,15 @@ getWindowStartupId (Display * dpy, Window window, char **startup_id)
 
     if (XGetTextProperty (dpy, window, &tp, net_startup_id))
     {
-        if ((tp.value) && (tp.encoding == XA_STRING) && (tp.format == 8)
-            && (tp.nitems != 0))
+        if (tp.value)
         {
-            *startup_id = strdup ((char *) tp.value);
+            if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
+            {
+                *startup_id = g_strdup ((char *) tp.value);
+                XFree (tp.value);
+                return TRUE;
+            }
             XFree (tp.value);
-            return TRUE;
         }
     }
 
