@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -346,7 +346,7 @@ grab_available (guint32 timestamp)
                 || (g2 != GDK_GRAB_SUCCESS))))
     {
         TRACE ("grab not available yet, waiting... (%i)", i);
-        usleep (100);
+        g_usleep (100);
         if (g1 != GDK_GRAB_SUCCESS)
         {
             g1 = gdk_pointer_grab (getGdkEventWindow (), TRUE, mask, NULL,
@@ -404,6 +404,10 @@ menu_popup (Menu * menu, int root_x, int root_y, int button,
             gdk_beep ();
             g_message (_("%s: GtkMenu failed to grab the pointer\n"),
                 g_get_prgname ());
+            gtk_menu_popdown (GTK_MENU (menu->menu));
+            menu_open = NULL;
+            popEventFilter ();
+            return FALSE;
         }
     }
     return TRUE;
