@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -105,7 +105,7 @@ createGC (Colormap cmap, char *col, int func, XFontStruct * font,
 }
 
 void
-sendClientMessage (Window w, Atom a, Atom x)
+sendClientMessage (Window w, Atom a, Atom x, Time timestamp)
 {
     XClientMessageEvent ev;
 
@@ -116,7 +116,7 @@ sendClientMessage (Window w, Atom a, Atom x)
     ev.message_type = a;
     ev.format = 32;
     ev.data.l[0] = x;
-    ev.data.l[1] = CurrentTime;
+    ev.data.l[1] = timestamp;
     XSendEvent (dpy, w, FALSE, 0L, (XEvent *)&ev);
 }
 
@@ -159,7 +159,6 @@ MyCheckWindow(Window w)
     Window dummy_root, parent;
     Window *wins = NULL;
     unsigned int count;
-    int dummy_x, dummy_y;
     Status test;
     
     g_return_val_if_fail (w != None, FALSE);
@@ -221,49 +220,3 @@ placeSidewalks(gboolean activate)
                           1, MyDisplayFullHeight (dpy, screen));
     }
 }
-
-Time
-stashEventTime (Time prevEventTime, XEvent * ev)
-{
-    Time newEventTime = CurrentTime;
-    Time lastEventTime = prevEventTime;
-
-    switch (ev->type)
-    {
-        case KeyPress:
-        case KeyRelease:
-            newEventTime = ev->xkey.time;
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            newEventTime = ev->xbutton.time;
-            break;
-        case MotionNotify:
-            newEventTime = ev->xmotion.time;
-            break;
-        case EnterNotify:
-        case LeaveNotify:
-            newEventTime = ev->xcrossing.time;
-            break;
-        case PropertyNotify:
-            newEventTime = ev->xproperty.time;
-            break;
-        case SelectionClear:
-            newEventTime = ev->xselectionclear.time;
-            break;
-        case SelectionRequest:
-            newEventTime = ev->xselectionrequest.time;
-            break;
-        case SelectionNotify:
-            newEventTime = ev->xselection.time;
-            break;
-        default:
-          return;
-    }
-    if ((newEventTime > CurrentTime) || ((CurrentTime - newEventTime) > 30000))
-    {
-        lastEventTime = newEventTime;
-    }
-    return lastEventTime;
-}
-
