@@ -1,8 +1,8 @@
 /*
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
-        the Free Software Foundation; You may only use version 2 of the License,
-        you have no option to use any other version.
+        the Free Software Foundation; either version 2, or (at your option)
+        any later version.
  
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,7 +32,7 @@
 
 void workspaceSwitch (int, Client *);
 void workspaceSetCount (int);
-void workspaceGetArea (CARD32 *, CARD32 *, Client *);
-void workspaceUpdateArea (CARD32 *, CARD32 *);
+void workspaceGetArea (int *, int *, Client *);
+void workspaceUpdateArea (int *, int *);
 
 #endif /* INC_WORKSPACES_H */
